#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Mar 11 17:37:33 2022

@author: pouriak
"""
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns
from statsmodels.miscmodels.ordinal_model import OrderedModel
from scipy.stats import norm
from sklearn.metrics import brier_score_loss as BS
import matplotlib as mpl
from bokeh.palettes import magma
from sklearn.metrics import top_k_accuracy_score as k_acc_score
from matplotlib.ticker import LinearLocator
import scipy.stats as st

mpl.rcParams['font.sans-serif'] = "Arial"


data=pd.read_excel('Data.xlsx',sheet_name='all')


data_arch=data[data['Story']>=8]


data_period=pd.read_excel('Data.xlsx',sheet_name='Arch')
data_period_highrise = data_period[data_period['Story']>=8]
Tn = data_period_highrise['Tn']

arch=data_arch['Arch']
#story=data_arch['Story']
gamma=1.25
    
    
SDR=data_arch['SDR']
Damage=data_arch['Damage']
Y=data_arch['Label']

pd.options.mode.chained_assignment = None
for i in range(len(SDR)):
    if SDR.iloc[i]<0.01:
        Damage.iloc[i]='Negligible'
        Y.iloc[i]=0
    elif (SDR.iloc[i]>=0.01) and (SDR.iloc[i]<0.026):
        Damage.iloc[i]='Slight'
        Y.iloc[i]=1
    elif (SDR.iloc[i]>=0.026) and (SDR.iloc[i]<0.036):
        Damage.iloc[i]='Moderate'
        Y.iloc[i]=2
    elif (SDR.iloc[i]>=0.036) and (SDR.iloc[i]<0.059/gamma):
        Damage.iloc[i]='Extensive'
        Y.iloc[i]=3
    else: 
        Damage.iloc[i]='Collapse' 
        Y.iloc[i]=4

var00= 'SA_1'
var0= 'SA_avg'
var1='SA_avg'; var2='Ds'; 
var1_m='SA_avg_2'; var2_m='SA_avg_3'; var3_m='Ds'

#var4='Ds'
# DS = pd.Series(Damage, dtype="category")
Y = Y.reset_index(drop=True)
SDR = SDR.reset_index(drop=True)
X_input_org = np.log(data_arch[[var0, var00]])
X_input = pd.concat([np.log(data_arch[[var1]]),(data_arch[[var2]])],axis=1)
X_input_m = pd.concat([(data_arch[[var1_m]]),data_arch[[var2_m]],(data_arch[[var3_m]])],axis=1)



X_org = X_input_org.reset_index(drop=True)
X = X_input.reset_index(drop=True)
X_m = X_input_m.reset_index(drop=True)


#% visualize inputs
plt.figure(figsize=(3.75,2.25), dpi=300)
sns.histplot(np.exp(X_input_org[var0]),stat="probability", bins=25, color='grey')
plt.xlim(0.0, 1.25)
plt.ylim(0, .2) 
plt.ylabel('Relative Frequency', fontsize=10)
plt.xlabel('$SA_{avg}$ (g)', fontsize=10)


plt.figure(figsize=(3.75,2.25), dpi=300)
sns.histplot(np.exp(X_input_org[var00]),stat="probability", bins=25, color='grey')
plt.xlim(0.0, 2.5)
plt.ylim(0, .2) 
plt.ylabel('Relative Frequency', fontsize=10)
plt.xlabel('$SA_{(T=1s)}$ (g)', fontsize=10)
#plt.title('Bivariate Fragility', fontsize=10)
#plt.xticks([0,.1,.2,.3,.4],fontsize=10)

#duration
plt.figure(figsize=(3.75,2.25), dpi=300)
sns.histplot(X_input[var2],stat="probability", bins=20, color='grey')
plt.xlim(0.0, 200)
plt.ylim(0, .25) 
plt.ylabel('Relative Frequency', fontsize=10)
plt.xlabel('$D_{s}$ (s)', fontsize=10)
plt.xticks([0,25,50,75,100,125,150,175,200],fontsize=10)

#period
plt.figure(figsize=(3.75,2.25), dpi=300)
sns.histplot(Tn,stat="probability", bins=8, color='grey')
plt.xlim(1, 5.5)
plt.ylim(0, .3) 
plt.ylabel('Relative Frequency', fontsize=10)
plt.xlabel('Period (s)', fontsize=10)
#plt.xticks([0,.1,.2,.3,.4,.5,.6,.7],fontsize=10)
#plt.title('Highrise Buildings', fontsize=10)

#%%
#training fragility models

a = pd.Categorical(Damage, categories=['Negligible','Slight','Moderate','Extensive','Collapse'], ordered=True)
DS = pd.Series(a)

from sklearn.model_selection import StratifiedKFold
skf = StratifiedKFold(n_splits=5)
all_metrics = [[]]
for train_index, test_index in skf.split(X,DS):
    
    X_train_org1,X_test_org1,DS_train_org1,DS_test_org1 = X_org.iloc[train_index],X_org.iloc[test_index],DS.iloc[train_index],DS.iloc[test_index]
    X_train1,X_test1,DS_train1,DS_test1 = X.iloc[train_index],X.iloc[test_index],DS.iloc[train_index],DS.iloc[test_index]
    X_train1_m,X_test1_m,DS_train1_m,DS_test1_m = X_m.iloc[train_index],X_m.iloc[test_index],DS.iloc[train_index],DS.iloc[test_index]

    Y_train1_m=Y[X_train1_m.index]; Y_test1_m=Y[X_test1_m.index]
    Y_train1=Y[X_train1.index]; Y_test1=Y[X_test1.index]
    Y_train_org1=Y[X_train_org1.index]; Y_test_org1=Y[X_test_org1.index]
    SDR_train1_m=SDR[X_train1_m.index]; SDR_test1_m=SDR[X_test1_m.index]
    SDR_train1=SDR[X_train1.index]; SDR_test1=SDR[X_test1.index]
    SDR_train_org1=SDR[X_train_org1.index]; SDR_test_org1=SDR[X_test_org1.index]
    
    X_train_org = X_train_org1.reset_index(drop=True); X_test_org = X_test_org1.reset_index(drop=True)
    DS_train_org = DS_train_org1.reset_index(drop=True); DS_test_org = DS_test_org1.reset_index(drop=True)
    
    X_train = X_train1.reset_index(drop=True); X_test = X_test1.reset_index(drop=True)
    DS_train = DS_train1.reset_index(drop=True); DS_test = DS_test1.reset_index(drop=True)
    
    X_train_m = X_train1_m.reset_index(drop=True); X_test_m = X_test1_m.reset_index(drop=True)
    DS_train_m = DS_train1_m.reset_index(drop=True); DS_test_m = DS_test1_m.reset_index(drop=True)
    
    Y_train_m = Y_train1_m.reset_index(drop=True); Y_test_m = Y_test1_m.reset_index(drop=True)
    Y_train = Y_train1.reset_index(drop=True); Y_test = Y_test1.reset_index(drop=True)
    Y_train_org = Y_train_org1.reset_index(drop=True); Y_test_org = Y_test_org1.reset_index(drop=True)
    SDR_train_m = SDR_train1_m.reset_index(drop=True); SDR_test_m = SDR_test1_m.reset_index(drop=True)
    SDR_train = SDR_train1.reset_index(drop=True); SDR_test = SDR_test1.reset_index(drop=True)
    SDR_train_org = SDR_train_org1.reset_index(drop=True); SDR_test_org = SDR_test_org1.reset_index(drop=True)


    mod_prob_org0 = OrderedModel(DS_train_org, X_train_org[var00], distr='probit')
    mod_prob_org = OrderedModel(DS_train_org, X_train_org[var0], distr='probit')
    mod_prob = OrderedModel(DS_train, X_train, distr='probit')
    mod_prob_m = OrderedModel(DS_train_m, X_train_m, distr='probit')
    
    res_prob_org0 = mod_prob_org0.fit(method='bfgs')
    res_prob_org = mod_prob_org.fit(method='bfgs')
    res_prob = mod_prob.fit(method='bfgs')
    res_prob_m = mod_prob_m.fit(method='bfgs')
    
    coeffs_org0 = res_prob_org0.params[:len(X_input_org[var00].transpose())]
    coeffs_org = res_prob_org.params[:len(X_input_org[var0].transpose())]
    coeffs = res_prob.params[:len(X_input.transpose())]
    coeffs_m = res_prob_m.params[:len(X_input_m.transpose())]
    print(res_prob_m.summary())


    num_of_thresholds = 4
    intcpt_org0 = mod_prob_org0.transform_threshold_params(res_prob_org0.params[-num_of_thresholds:])
    intcpt_org = mod_prob_org.transform_threshold_params(res_prob_org.params[-num_of_thresholds:])
    intcpt = mod_prob.transform_threshold_params(res_prob.params[-num_of_thresholds:])
    intcpt_m = mod_prob_m.transform_threshold_params(res_prob_m.params[-num_of_thresholds:])
    
    
    def p1_0(x1):
        #Prob_DS0: probability of observing negligible damage
        Prob_DS0 = norm.cdf(intcpt_org0[1]-x1*coeffs_org0[0])
        Prob_DS1 = norm.cdf(intcpt_org0[2]-x1*coeffs_org0[0]) - norm.cdf(intcpt_org0[1]-x1*coeffs_org0[0])
        Prob_DS2 = norm.cdf(intcpt_org0[3]-x1*coeffs_org0[0]) - norm.cdf(intcpt_org0[2]-x1*coeffs_org0[0])
        Prob_DS3 = norm.cdf(intcpt_org0[4]-x1*coeffs_org0[0]) - norm.cdf(intcpt_org0[3]-x1*coeffs_org0[0])
        Prob_DS4 = 1 - norm.cdf(intcpt_org0[4]-x1*coeffs_org0[0])
        #probability of exceedance 
        #Prob_exc_DS5 = Prob_DS5
        Prob_exc_DS4 = Prob_DS4
        Prob_exc_DS3 = Prob_DS4+Prob_DS3
        Prob_exc_DS2 = Prob_DS4+Prob_DS3+Prob_DS2
        Prob_exc_DS1 = Prob_DS4+Prob_DS3+Prob_DS2+Prob_DS1
        return [Prob_DS0, Prob_DS1, Prob_DS2, Prob_DS3, Prob_DS4], [Prob_exc_DS1, Prob_exc_DS2, Prob_exc_DS3, Prob_exc_DS4]
    
    def p1(x1):
        #Prob_DS0: probability of observing negligible damage
        Prob_DS0 = norm.cdf(intcpt_org[1]-x1*coeffs_org[0])
        Prob_DS1 = norm.cdf(intcpt_org[2]-x1*coeffs_org[0]) - norm.cdf(intcpt_org[1]-x1*coeffs_org[0])
        Prob_DS2 = norm.cdf(intcpt_org[3]-x1*coeffs_org[0]) - norm.cdf(intcpt_org[2]-x1*coeffs_org[0])
        Prob_DS3 = norm.cdf(intcpt_org[4]-x1*coeffs_org[0]) - norm.cdf(intcpt_org[3]-x1*coeffs_org[0])
        Prob_DS4 = 1 - norm.cdf(intcpt_org[4]-x1*coeffs_org[0])
        #probability of exceedance 
        #Prob_exc_DS5 = Prob_DS5
        Prob_exc_DS4 = Prob_DS4
        Prob_exc_DS3 = Prob_DS4+Prob_DS3
        Prob_exc_DS2 = Prob_DS4+Prob_DS3+Prob_DS2
        Prob_exc_DS1 = Prob_DS4+Prob_DS3+Prob_DS2+Prob_DS1
        return [Prob_DS0, Prob_DS1, Prob_DS2, Prob_DS3, Prob_DS4], [Prob_exc_DS1, Prob_exc_DS2, Prob_exc_DS3, Prob_exc_DS4]
    
    def p2(x1,x2):
    
        Prob_DS0 = norm.cdf(intcpt[1]-x1*coeffs[0]-x2*coeffs[1]) - norm.cdf(intcpt[0]-x1*coeffs[0]-x2*coeffs[1])
        Prob_DS1 = norm.cdf(intcpt[2]-x1*coeffs[0]-x2*coeffs[1]) - norm.cdf(intcpt[1]-x1*coeffs[0]-x2*coeffs[1])
        Prob_DS2 = norm.cdf(intcpt[3]-x1*coeffs[0]-x2*coeffs[1]) - norm.cdf(intcpt[2]-x1*coeffs[0]-x2*coeffs[1])
        Prob_DS3 = norm.cdf(intcpt[4]-x1*coeffs[0]-x2*coeffs[1]) - norm.cdf(intcpt[3]-x1*coeffs[0]-x2*coeffs[1])
        Prob_DS4 = norm.cdf(intcpt[5]-x1*coeffs[0]-x2*coeffs[1]) - norm.cdf(intcpt[4]-x1*coeffs[0]-x2*coeffs[1])
        #probability of exceedance 
        Prob_exc_DS4 = Prob_DS4
        Prob_exc_DS3 = Prob_DS4+Prob_DS3
        Prob_exc_DS2 = Prob_DS4+Prob_DS3+Prob_DS2
        Prob_exc_DS1 = Prob_DS4+Prob_DS3+Prob_DS2+Prob_DS1
        return [Prob_DS0, Prob_DS1, Prob_DS2, Prob_DS3, Prob_DS4], [Prob_exc_DS1, Prob_exc_DS2, Prob_exc_DS3, Prob_exc_DS4]
    
    def p3(x1,x2,x3):
    
        Prob_DS0 = norm.cdf(intcpt_m[1]-x1*coeffs_m[0]-x2*coeffs_m[1]-x3*coeffs_m[2]) - norm.cdf(intcpt_m[0]-x1*coeffs_m[0]-x2*coeffs_m[1]-x3*coeffs_m[2])
        Prob_DS1 = norm.cdf(intcpt_m[2]-x1*coeffs_m[0]-x2*coeffs_m[1]-x3*coeffs_m[2]) - norm.cdf(intcpt_m[1]-x1*coeffs_m[0]-x2*coeffs_m[1]-x3*coeffs_m[2])
        Prob_DS2 = norm.cdf(intcpt_m[3]-x1*coeffs_m[0]-x2*coeffs_m[1]-x3*coeffs_m[2]) - norm.cdf(intcpt_m[2]-x1*coeffs_m[0]-x2*coeffs_m[1]-x3*coeffs_m[2])
        Prob_DS3 = norm.cdf(intcpt_m[4]-x1*coeffs_m[0]-x2*coeffs_m[1]-x3*coeffs_m[2]) - norm.cdf(intcpt_m[3]-x1*coeffs_m[0]-x2*coeffs_m[1]-x3*coeffs_m[2])
        Prob_DS4 = norm.cdf(intcpt_m[5]-x1*coeffs_m[0]-x2*coeffs_m[1]-x3*coeffs_m[2]) - norm.cdf(intcpt_m[4]-x1*coeffs_m[0]-x2*coeffs_m[1]-x3*coeffs_m[2])
        #probability of exceedance 
        Prob_exc_DS4 = Prob_DS4
        Prob_exc_DS3 = Prob_DS4+Prob_DS3
        Prob_exc_DS2 = Prob_DS4+Prob_DS3+Prob_DS2
        Prob_exc_DS1 = Prob_DS4+Prob_DS3+Prob_DS2+Prob_DS1
        return [Prob_DS0, Prob_DS1, Prob_DS2, Prob_DS3, Prob_DS4], [Prob_exc_DS1, Prob_exc_DS2, Prob_exc_DS3, Prob_exc_DS4]
    
    
    for i in range(len(SDR)):
        if SDR.iloc[i]<0.01:
            Damage.iloc[i]='Negligible'
            Y.iloc[i]=0
        elif (SDR.iloc[i]>=0.01) and (SDR.iloc[i]<0.026):
            Damage.iloc[i]='Slight'
            Y.iloc[i]=1
        elif (SDR.iloc[i]>=0.026) and (SDR.iloc[i]<0.036):
            Damage.iloc[i]='Moderate'
            Y.iloc[i]=2
        elif (SDR.iloc[i]>=0.036) and (SDR.iloc[i]<0.059/gamma):
            Damage.iloc[i]='Extensive'
            Y.iloc[i]=3
        else: 
            Damage.iloc[i]='Collapse' 
            Y.iloc[i]=4
    
    def q(y):
        if y==0:
            p_obs=[1,0,0,0,0]
        elif y==1:
            p_obs=[0,1,0,0,0]
        elif y==2:
            p_obs=[0,0,1,0,0]
        elif y==3:
            p_obs=[0,0,0,1,0]
        else:
            p_obs=[0,0,0,0,1]
        return p_obs
    
    
    BS_array00 = np.zeros(len(X_test_org[var00]))
    BS_array0 = np.zeros(len(X_test_org[var0]))
    BS_array1 = np.zeros(len(X_test))
    BS_array2 = np.zeros(len(X_test_m))
    
    BS_array_all_m = np.zeros(len(X_m))
    BS_array_all = np.zeros(len(X))
    BS_array_all_uni = np.zeros(len(X))
    BS_array_all_uni0 = np.zeros(len(X))
    np.seterr(divide = 'ignore') 
    np.errstate(invalid='ignore')
    
    probs00=[]
    probs0=[]
    probs1=[]
    probs2=[]
    probs_obs=[]
    probs_obs_m=[]
      
    probs_obs_all=[]
    probs_est_all_m=[]; 
    probs_est_all=[];
    probs_est_all_uni=[];
    probs_est_all_uni0=[];
    
    
    for i in range(len(X_test)):
        p1_0_estimate = p1_0(X_test_org[var00][i])[0]
        probs00.append(p1_0_estimate)
        
        p1_estimate = p1(X_test_org[var0][i])[0]
        probs0.append(p1_estimate)
        
        p2_estimate = p2(X_test[var1][i],X_test[var2][i])[0]
        probs1.append(p2_estimate)
        
        p3_estimate = p3(X_test_m[var1_m][i],X_test_m[var2_m][i],X_test_m[var3_m][i])[0]
        probs2.append(p3_estimate)
        
        q_estimate_org = q(Y_test_org[i]) 
        q_estimate = q(Y_test[i])
        q_estimate_m = q(Y_test_m[i])
        
        probs_obs.append(q_estimate)
        probs_obs_m.append(q_estimate_m)
        
        BS_array00[i] = BS(q_estimate_org, p1_0_estimate)
        BS_array0[i] = BS(q_estimate_org, p1_estimate)
        BS_array1[i] = BS(q_estimate, p2_estimate)
        BS_array2[i] = BS(q_estimate_m, p3_estimate)
    
    BS_avg00 = np.mean(BS_array00)
    BS_avg0 = np.mean(BS_array0)
    BS_avg1 = np.mean(BS_array1)
    BS_avg2 = np.mean(BS_array2)
    
    
    #% nominal probit regression
    #import statsmodels.api as sm
    from statsmodels.discrete.discrete_model import Probit
    Y_probit4_m = Y_train_m >=4 ; Y_probit3_m = Y_train_m >=3; Y_probit2_m = Y_train_m >=2; Y_probit1_m = Y_train_m >=1; 
    Y_probit4 = Y_train >=4 ; Y_probit3 = Y_train >=3; Y_probit2 = Y_train >=2; Y_probit1 = Y_train >=1; 
    Y_probit4_org = Y_train_org >=4 ; Y_probit3_org = Y_train_org >=3; Y_probit2_org = Y_train_org >=2; Y_probit1_org = Y_train_org >=1; 
    
    
    const=pd.DataFrame(np.ones(len(X_train)))
    X_train2_m = pd.concat([X_train_m,const], axis=1)
    X_train2 = pd.concat([X_train,const], axis=1)
    X_train2_org = pd.concat([X_train_org[var0],const], axis=1)
    X_train2_org0 = pd.concat([X_train_org[var00],const], axis=1)
    
    
    model4_m = Probit(Y_probit4_m, X_train2_m); model3_m = Probit(Y_probit3_m, X_train2_m); model2_m = Probit(Y_probit2_m, X_train2_m); model1_m = Probit(Y_probit1_m, X_train2_m);
    model4 = Probit(Y_probit4, X_train2); model3 = Probit(Y_probit3, X_train2); model2 = Probit(Y_probit2, X_train2); model1 = Probit(Y_probit1, X_train2);
    model4_org = Probit(Y_probit4_org, X_train2_org); model3_org = Probit(Y_probit3_org, X_train2_org); model2_org = Probit(Y_probit2_org, X_train2_org); model1_org = Probit(Y_probit1_org, X_train2_org);
    model4_org0 = Probit(Y_probit4_org, X_train2_org0); model3_org0 = Probit(Y_probit3_org, X_train2_org0); model2_org0 = Probit(Y_probit2_org, X_train2_org0); model1_org0 = Probit(Y_probit1_org, X_train2_org0);
    
    probit_model4_m = model4_m.fit(); probit_model3_m = model3_m.fit(); probit_model2_m = model2_m.fit(); probit_model1_m = model1_m.fit(); 
    probit_model4 = model4.fit(); probit_model3 = model3.fit(); probit_model2 = model2.fit(); probit_model1 = model1.fit(); 
    probit_model4_org = model4_org.fit(); probit_model3_org = model3_org.fit(); probit_model2_org = model2_org.fit(); probit_model1_org = model1_org.fit(); 
    probit_model4_org0 = model4_org0.fit(); probit_model3_org0 = model3_org0.fit(); probit_model2_org0 = model2_org0.fit(); probit_model1_org0 = model1_org0.fit(); 
    
    
    coeff4_m=probit_model4_m.params; coeff3_m=probit_model3_m.params; coeff2_m=probit_model2_m.params; coeff1_m=probit_model1_m.params; 
    coeff4=probit_model4.params; coeff3=probit_model3.params; coeff2=probit_model2.params; coeff1=probit_model1.params; 
    coeff4_org=probit_model4_org.params; coeff3_org=probit_model3_org.params; coeff2_org=probit_model2_org.params; coeff1_org=probit_model1_org.params; 
    coeff4_org0=probit_model4_org0.params; coeff3_org0=probit_model3_org0.params; coeff2_org0=probit_model2_org0.params; coeff1_org0=probit_model1_org0.params; 
    
    def probit_uni0(x1):
        Prob_DS1 = norm.cdf(coeff1_org0[0] + x1*coeff1_org0[var00])
        Prob_DS2 = norm.cdf(coeff2_org0[0] + x1*coeff2_org0[var00])
        Prob_DS3 = norm.cdf(coeff3_org0[0] + x1*coeff3_org0[var00])
        Prob_DS4 = norm.cdf(coeff4_org0[0] + x1*coeff4_org0[var00])
        #probability of occurence 
        Prob_occ_DS0 = (1-Prob_DS1)
        Prob_occ_DS1 = (-Prob_DS2+Prob_DS1)
        Prob_occ_DS2 = (-Prob_DS3+Prob_DS2)
        Prob_occ_DS3 = (-Prob_DS4+Prob_DS3)
        Prob_occ_DS4 = (Prob_DS4)
        
        #probability of exceedance 
        Prob_exc_DS4 = Prob_occ_DS4
        Prob_exc_DS3 = Prob_occ_DS4+Prob_occ_DS3
        Prob_exc_DS2 = Prob_occ_DS4+Prob_occ_DS3+Prob_occ_DS2
        Prob_exc_DS1 = Prob_occ_DS4+Prob_occ_DS3+Prob_occ_DS2+Prob_occ_DS1
        return [abs(Prob_occ_DS0), abs(Prob_occ_DS1), abs(Prob_occ_DS2), abs(Prob_occ_DS3), abs(Prob_occ_DS4)], [Prob_exc_DS1, Prob_exc_DS2, Prob_exc_DS3, Prob_exc_DS4]
    
    def probit_uni(x1):
        Prob_DS1 = norm.cdf(coeff1_org[0] + x1*coeff1_org[var0])
        Prob_DS2 = norm.cdf(coeff2_org[0] + x1*coeff2_org[var0])
        Prob_DS3 = norm.cdf(coeff3_org[0] + x1*coeff3_org[var0])
        Prob_DS4 = norm.cdf(coeff4_org[0] + x1*coeff4_org[var0])
        #probability of occurence 
        Prob_occ_DS0 = (1-Prob_DS1)
        Prob_occ_DS1 = (-Prob_DS2+Prob_DS1)
        Prob_occ_DS2 = (-Prob_DS3+Prob_DS2)
        Prob_occ_DS3 = (-Prob_DS4+Prob_DS3)
        Prob_occ_DS4 = (Prob_DS4)
        
        #probability of exceedance 
        Prob_exc_DS4 = Prob_occ_DS4
        Prob_exc_DS3 = Prob_occ_DS4+Prob_occ_DS3
        Prob_exc_DS2 = Prob_occ_DS4+Prob_occ_DS3+Prob_occ_DS2
        Prob_exc_DS1 = Prob_occ_DS4+Prob_occ_DS3+Prob_occ_DS2+Prob_occ_DS1
        return [abs(Prob_occ_DS0), abs(Prob_occ_DS1), abs(Prob_occ_DS2), abs(Prob_occ_DS3), abs(Prob_occ_DS4)], [Prob_exc_DS1, Prob_exc_DS2, Prob_exc_DS3, Prob_exc_DS4]
    
    
    def probit2(x1,x2):
        Prob_DS1 = norm.cdf(coeff1[0] + x1*coeff1[var1] + x2*coeff1[var2])
        Prob_DS2 = norm.cdf(coeff2[0] + x1*coeff2[var1] + x2*coeff2[var2])
        Prob_DS3 = norm.cdf(coeff3[0] + x1*coeff3[var1] + x2*coeff3[var2])
        Prob_DS4 = norm.cdf(coeff4[0] + x1*coeff4[var1] + x2*coeff4[var2])
        #probability of occurence 
        Prob_occ_DS0 = (1-Prob_DS1)
        Prob_occ_DS1 = (-Prob_DS2+Prob_DS1)
        Prob_occ_DS2 = (-Prob_DS3+Prob_DS2)
        Prob_occ_DS3 = (-Prob_DS4+Prob_DS3)
        Prob_occ_DS4 = (Prob_DS4)
        #probability of exceedance 
        Prob_exc_DS4 = Prob_occ_DS4
        Prob_exc_DS3 = Prob_occ_DS4+Prob_occ_DS3
        Prob_exc_DS2 = Prob_occ_DS4+Prob_occ_DS3+Prob_occ_DS2
        Prob_exc_DS1 = Prob_occ_DS4+Prob_occ_DS3+Prob_occ_DS2+Prob_occ_DS1
        return [abs(Prob_occ_DS0), abs(Prob_occ_DS1), abs(Prob_occ_DS2), abs(Prob_occ_DS3), abs(Prob_occ_DS4)], [Prob_exc_DS1, Prob_exc_DS2, Prob_exc_DS3, Prob_exc_DS4]
    
    
    def probit3(x1,x2,x3):
        Prob_DS1 = norm.cdf(coeff1_m[0] + x1*coeff1_m[var1_m] + x2*coeff1_m[var2_m] + x3*coeff1_m[var3_m])
        Prob_DS2 = norm.cdf(coeff2_m[0] + x1*coeff2_m[var1_m] + x2*coeff2_m[var2_m] + x3*coeff2_m[var3_m])
        Prob_DS3 = norm.cdf(coeff3_m[0] + x1*coeff3_m[var1_m] + x2*coeff3_m[var2_m] + x3*coeff3_m[var3_m])
        Prob_DS4 = norm.cdf(coeff4_m[0] + x1*coeff4_m[var1_m] + x2*coeff4_m[var2_m] + x3*coeff4_m[var3_m])
        #probability of occurence 
        Prob_occ_DS0 = (1-Prob_DS1)
        Prob_occ_DS1 = (-Prob_DS2+Prob_DS1)
        Prob_occ_DS2 = (-Prob_DS3+Prob_DS2)
        Prob_occ_DS3 = (-Prob_DS4+Prob_DS3)
        Prob_occ_DS4 = (Prob_DS4)
        #probability of exceedance 
        Prob_exc_DS4 = Prob_occ_DS4
        Prob_exc_DS3 = Prob_occ_DS4+Prob_occ_DS3
        Prob_exc_DS2 = Prob_occ_DS4+Prob_occ_DS3+Prob_occ_DS2
        Prob_exc_DS1 = Prob_occ_DS4+Prob_occ_DS3+Prob_occ_DS2+Prob_occ_DS1
        return [abs(Prob_occ_DS0), abs(Prob_occ_DS1), abs(Prob_occ_DS2), abs(Prob_occ_DS3), abs(Prob_occ_DS4)], [Prob_exc_DS1, Prob_exc_DS2, Prob_exc_DS3, Prob_exc_DS4]
    
    
    BS_array_probit_m = np.zeros(len(X_test))
    BS_array_probit = np.zeros(len(X_test))
    BS_array_probit_uni = np.zeros(len(X_test))
    BS_array_probit_uni0 = np.zeros(len(X_test))
    
    probs_p_m=[]
    probs_p=[]
    probs_p_uni=[]
    probs_p_uni0=[]
    
    
    for i in range(len(X_test)):    
        probit_estimate_m = probit3(X_test_m[var1_m][i],X_test_m[var2_m][i],X_test_m[var3_m][i])[0]
        probit_estimate = probit2(X_test[var1][i],X_test[var2][i])[0]
        probit_uni_estimate = probit_uni(X_test_org[var0][i])[0]
        probit_uni_estimate0 = probit_uni0(X_test_org[var00][i])[0]
        probs_p_m.append(probit_estimate_m)
        probs_p.append(probit_estimate)
        probs_p_uni.append(probit_uni_estimate)
        probs_p_uni0.append(probit_uni_estimate0)
        
        q_estimate_m = q(Y_test_m[i])
        q_estimate = q(Y_test[i])
        q_estimate_org = q(Y_test_org[i])
        #probs_obs.append(q_estimate)
        
        BS_array_probit_m[i] = BS(q_estimate_m, probit_estimate_m)
        BS_array_probit[i] = BS(q_estimate, probit_estimate)
        BS_array_probit_uni[i] = BS(q_estimate_org, probit_uni_estimate)
        BS_array_probit_uni0[i] = BS(q_estimate_org, probit_uni_estimate0)
    
    BS_avg_probit_m = np.mean(BS_array_probit_m)
    BS_avg_probit = np.mean(BS_array_probit)
    BS_avg_probit_uni = np.mean(BS_array_probit_uni)
    BS_avg_probit_uni0 = np.mean(BS_array_probit_uni0)
    #print(probit_model4.summary())
    
    print("BS of nominal univariate (T=1) probit function is",BS_avg_probit_uni0)
    print("BS of nominal univariate (SA_avg) probit function is",BS_avg_probit_uni)
    print("BS of ordinal univariate (SA_avg) probit function is",BS_avg0)
    print("BS of nominal bivariate probit function is",BS_avg_probit)
    print("BS of ordinal bivariate probit function is",BS_avg1)
    print("BS of nominal multivariate probit function is",BS_avg_probit_m)
    print("BS of ordinal multivariate probit function is",BS_avg2)
        
    accuracy_ordinal_multi = k_acc_score(Y_test_m,probs2,k=1)
    accuracy_ordinal_bi = k_acc_score(Y_test,probs1,k=1)
    accuracy_ordinal_uni = k_acc_score(Y_test_org,probs0,k=1)
    accuracy_nominal_multi = k_acc_score(Y_test_m,probs_p_m,k=1)
    accuracy_nominal_bi = k_acc_score(Y_test,probs_p,k=1)
    accuracy_nominal_uni = k_acc_score(Y_test_org,probs_p_uni,k=1)
    accuracy_nominal_uni0 = k_acc_score(Y_test_org,probs_p_uni0,k=1)
    
    k_score_ordinal_multi = k_acc_score(Y_test_m,probs2)
    k_score_ordinal_bi = k_acc_score(Y_test,probs1)
    k_score_ordinal_uni = k_acc_score(Y_test_org,probs0)
    k_score_nominal_multi = k_acc_score(Y_test_m,probs_p_m)
    k_score_nominal_bi = k_acc_score(Y_test,probs_p)
    k_score_nominal_uni = k_acc_score(Y_test_org,probs_p_uni)
    k_score_nominal_uni0 = k_acc_score(Y_test_org,probs_p_uni0)
    
    
    #probability of misclassification 
    prob_misc_nominal_uni0 = np.zeros((len(X_test),1))
    prob_misc_nominal_uni = np.zeros((len(X_test),1))
    prob_misc_nominal_bi = np.zeros((len(X_test),1))
    prob_misc_nominal_multi = np.zeros((len(X_test),1))
    prob_misc_ordinal_uni = np.zeros((len(X_test),1))
    prob_misc_ordinal_bi = np.zeros((len(X_test),1))
    prob_misc_ordinal_multi = np.zeros((len(X_test),1))
    
    for i in range(len(Y_test)):
        prob_misc_nominal_uni0[i] = 1-probs_p_uni0[i][int(Y_test_org[i])]
        prob_misc_nominal_uni[i] = 1-probs_p_uni[i][int(Y_test_org[i])]
        prob_misc_nominal_bi[i] = 1-probs_p[i][int(Y_test[i])]
        prob_misc_nominal_multi[i] = 1-probs_p_m[i][int(Y_test_m[i])]
    
        prob_misc_ordinal_multi[i] = 1-probs2[i][int(Y_test_m[i])]
        prob_misc_ordinal_bi[i] = 1-probs1[i][int(Y_test[i])]
        prob_misc_ordinal_uni[i] = 1-probs0[i][int(Y_test[i])]
    
    col_indx_uni = Y_test_org[Y_test_org==4].index.tolist()
    col_indx_bi = Y_test[Y_test==4].index.tolist()
    col_indx_multi = Y_test_m[Y_test_m==4].index.tolist()
    
    prob_col_misc_uni0 = np.mean(prob_misc_nominal_uni0[[col_indx_uni]])
    prob_col_misc_uni = np.mean(prob_misc_nominal_uni[[col_indx_uni]])
    prob_col_misc_uni_ordinal = np.mean(prob_misc_ordinal_uni[[col_indx_uni]])
    prob_col_misc_bi = np.mean(prob_misc_nominal_bi[[col_indx_bi]])
    prob_col_misc_bi_ordinal = np.mean(prob_misc_ordinal_bi[[col_indx_bi]])
    prob_col_misc_multi = np.mean(prob_misc_nominal_multi[[col_indx_multi]])
    prob_col_misc_multi_ordinal = np.mean(prob_misc_ordinal_multi[[col_indx_multi]])
    
   
    all_metrics.append(np.squeeze([[BS_avg_probit_uni0,BS_avg_probit_uni,BS_avg_probit,BS_avg_probit_m,BS_avg2],
                              [accuracy_nominal_uni0,accuracy_nominal_uni,accuracy_nominal_bi,accuracy_nominal_multi,accuracy_ordinal_multi],
                              [k_score_nominal_uni0,k_score_nominal_uni,k_score_nominal_bi,k_score_nominal_multi,k_score_ordinal_multi],
                              [prob_col_misc_uni0,prob_col_misc_uni,prob_col_misc_bi,prob_col_misc_multi,prob_col_misc_multi_ordinal]]).T)
    
#%%
read_file=pd.ExcelFile('results_summary.xlsx')
data_res=pd.read_excel(read_file,'Sheet1')
BS_score = data_res.iloc[0:5,6]
BS_error = [data_res.iloc[12:17,6], data_res.iloc[6:11,6]]

acc_score = data_res.iloc[0:5,7]
acc_error = [data_res.iloc[12:17,7], data_res.iloc[6:11,7]]

acc2_score = data_res.iloc[0:5,8]
acc2_error = [data_res.iloc[12:17,8], data_res.iloc[6:11,8]]

col_score = data_res.iloc[0:5,9]
col_error = [data_res.iloc[12:17,9], data_res.iloc[6:11,9]]

colors = [magma(24)[23], magma(24)[20], magma(24)[16], magma(24)[13], magma(24)[4]]


model = ['U$_{1}$','U$_{2}$','B$_{n}$','M$_{n}$','M$_{o}$']
plt.figure(figsize=(3.25,2), dpi=300)
plt.bar(model, BS_score, yerr=BS_error, align='center', color=colors, alpha=1, ecolor='k', capsize=5)
#plt.ylim(0.07, .12)               
plt.xlabel('Model', fontsize=9)
plt.ylabel('Brier Score', fontsize=9)
plt.yticks(fontsize=9)

plt.figure(figsize=(3.25,2), dpi=300)
plt.bar(model, col_score, yerr=col_error, align='center', color=colors, alpha=1, ecolor='black', capsize=5)
#plt.ylim(0.4, 1)               
plt.xlabel('Model', fontsize=9)
#plt.yticks([0.4,0.5,0.6,0.7,0.8,0.9,1], fontsize=9)
plt.ylabel('Collapse Misclassification', fontsize=9)
plt.yticks(fontsize=9)


plt.figure(figsize=(3.25,2), dpi=300)
plt.bar(model, acc_score, yerr=acc_error, align='center', color=colors, alpha=1, ecolor='black', capsize=5)
plt.ylim(0.5, 1)               
plt.xlabel('Model', fontsize=9)
#plt.yticks([.5,0.55,0.6,.65,0.7], fontsize=9)
plt.ylabel('Accuracy', fontsize=9)
plt.yticks(fontsize=9)

plt.figure(figsize=(3.25,2), dpi=300)
plt.bar(model, acc2_score, yerr=acc2_error, align='center', color=colors, alpha=1, ecolor='black', capsize=5)
plt.ylim(0.7, 1)               
plt.xlabel('Model', fontsize=9)
#plt.yticks([0.75,0.8,.85,.9,0.95], fontsize=9)
plt.ylabel('Top-2 Accuracy', fontsize=9)
plt.yticks(fontsize=9)


#%%
probs_est_all_p_m=[]
probs_est_all_p=[]
probs_est_all_p_uni=[]
probs_est_all_p_uni0=[]

BS_array_all_p_m = np.zeros(len(X))
BS_array_all_p = np.zeros(len(X))
BS_array_all_p_uni = np.zeros(len(X))
BS_array_all_p_uni0 = np.zeros(len(X))

# test nominal functions on the entire dataset
for i in range(len(X)):
    p_estimate_m = probit3(X_m[var1_m][i],X_m[var2_m][i],X_m[var3_m][i])[0]
    p_estimate = probit2(X[var1][i],X[var2][i])[0]
    p_estimate_uni = probit_uni(X_org[var0][i])[0]
    p_estimate_uni0 = probit_uni0(X_org[var00][i])[0]
    probs_est_all_p_m.append(p_estimate_m)
    probs_est_all_p.append(p_estimate)
    probs_est_all_p_uni.append(p_estimate_uni)
    probs_est_all_p_uni0.append(p_estimate_uni0)
    
    q_estimate = q(Y[i])
    #probs_obs_all.append(q_estimate)
    BS_array_all_p_m[i] = BS(q_estimate, p_estimate_m)
    BS_array_all_p[i] = BS(q_estimate, p_estimate)
    BS_array_all_p_uni[i] = BS(q_estimate, p_estimate_uni)
    BS_array_all_p_uni0[i] = BS(q_estimate, p_estimate_uni0)


# test ordinal functions on the entire dataset
for i in range(len(X)):
    p_estimate_m = p3(X_m[var1_m][i],X_m[var2_m][i],X_m[var3_m][i])[0]
    p_estimate = p2(X[var1][i],X[var2][i])[0]
    p_estimate_uni = p1(X_org[var0][i])[0]
    p_estimate_uni0 = p1_0(X_org[var00][i])[0]
    probs_est_all_m.append(p_estimate_m)
    probs_est_all.append(p_estimate)
    probs_est_all_uni.append(p_estimate_uni)
    probs_est_all_uni0.append(p_estimate_uni0)
    
    q_estimate = q(Y[i])
    probs_obs_all.append(q_estimate)
    
    BS_array_all_m[i] = BS(q_estimate, p_estimate_m)
    BS_array_all[i] = BS(q_estimate, p_estimate)
    BS_array_all_uni[i] = BS(q_estimate, p_estimate_uni)
    BS_array_all_uni0[i] = BS(q_estimate, p_estimate_uni0)



#%%
#% compute expected loss ratio

loss_array = np.array([0, 0.05, 0.2, 0.6, 1])

ELR_o = np.dot(np.squeeze(probs_obs_all), loss_array);  ELR_o_avg_all = np.mean(ELR_o) #observed

#ordinal
ELR_u0 = np.dot(np.squeeze(probs_est_all_uni0), loss_array);  ELR_u_avg_all0 = np.mean(ELR_u0) #univariate (T=1)
ELR_u = np.dot(np.squeeze(probs_est_all_uni), loss_array);  ELR_u_avg_all = np.mean(ELR_u) #univariate
ELR_b = np.dot(np.squeeze(probs_est_all), loss_array);  ELR_b_avg_all = np.mean(ELR_b) #bivariate
ELR_m = np.dot(np.squeeze(probs_est_all_m), loss_array);  ELR_m_avg_all = np.mean(ELR_m) #multivariate

#nominal
ELR_up0 = np.dot(np.squeeze(probs_est_all_p_uni0), loss_array);  ELR_up_avg_all0 = np.mean(ELR_up0) #univariate (T=1)
ELR_up = np.dot(np.squeeze(probs_est_all_p_uni), loss_array);  ELR_up_avg_all = np.mean(ELR_up) #univariate
ELR_bp = np.dot(np.squeeze(probs_est_all_p), loss_array);  ELR_bp_avg_all = np.mean(ELR_bp) #bivariate
ELR_mp = np.dot(np.squeeze(probs_est_all_p_m), loss_array);  ELR_mp_avg_all = np.mean(ELR_mp) #multivariate


ELR_100up=[]; ELR_100up0=[]; ELR_100bp=[]; ELR_100b=[]; ELR_100o=[]; ELR_100m=[]; ELR_100mp=[]
ELR_475up=[]; ELR_475up0=[]; ELR_475bp=[]; ELR_475b=[]; ELR_475o=[]; ELR_475m=[]; ELR_475mp=[]
ELR_975up=[]; ELR_975up0=[]; ELR_975bp=[]; ELR_975b=[]; ELR_975o=[]; ELR_975m=[]; ELR_975mp=[]
ELR_2475up=[]; ELR_2475up0=[]; ELR_2475bp=[]; ELR_2475b=[]; ELR_2475o=[]; ELR_2475m=[]; ELR_2475mp=[]
ELR_4975up=[]; ELR_4975up0=[]; ELR_4975bp=[]; ELR_4975b=[]; ELR_4975o=[]; ELR_4975m=[]; ELR_4975mp=[]

for i in range(len(np.unique(data_arch['Arch']))):
    a1=data_arch[data_arch['Arch']== np.unique(data_arch['Arch'])[i]]
    ELR_100up.append(np.mean(ELR_up[a1[a1['SA_Tn']==np.unique(a1['SA_Tn'])[0]].index])) ; ELR_100up0.append(np.mean(ELR_up0[a1[a1['SA_Tn']==np.unique(a1['SA_Tn'])[0]].index])); ELR_100bp.append(np.mean(ELR_bp[a1[a1['SA_Tn']==np.unique(a1['SA_Tn'])[0]].index])); ELR_100b.append(np.mean(ELR_b[a1[a1['SA_Tn']==np.unique(a1['SA_Tn'])[0]].index])); ELR_100o.append(np.mean(ELR_o[a1[a1['SA_Tn']==np.unique(a1['SA_Tn'])[0]].index])); ELR_100m.append(np.mean(ELR_m[a1[a1['SA_Tn']==np.unique(a1['SA_Tn'])[0]].index])); ELR_100mp.append(np.mean(ELR_mp[a1[a1['SA_Tn']==np.unique(a1['SA_Tn'])[0]].index]))  
    
    ELR_475up.append(np.mean(ELR_up[a1[a1['SA_Tn']==np.unique(a1['SA_Tn'])[1]].index])) ; ELR_475up0.append(np.mean(ELR_up0[a1[a1['SA_Tn']==np.unique(a1['SA_Tn'])[1]].index])); ELR_475bp.append(np.mean(ELR_bp[a1[a1['SA_Tn']==np.unique(a1['SA_Tn'])[1]].index])); ELR_475b.append(np.mean(ELR_b[a1[a1['SA_Tn']==np.unique(a1['SA_Tn'])[1]].index])); ELR_475o.append(np.mean(ELR_o[a1[a1['SA_Tn']==np.unique(a1['SA_Tn'])[1]].index])); ELR_475m.append(np.mean(ELR_m[a1[a1['SA_Tn']==np.unique(a1['SA_Tn'])[1]].index])); ELR_475mp.append(np.mean(ELR_mp[a1[a1['SA_Tn']==np.unique(a1['SA_Tn'])[1]].index])); 
    ELR_975up.append(np.mean(ELR_up[a1[a1['SA_Tn']==np.unique(a1['SA_Tn'])[2]].index])) ; ELR_975up0.append(np.mean(ELR_up0[a1[a1['SA_Tn']==np.unique(a1['SA_Tn'])[2]].index])); ELR_975bp.append(np.mean(ELR_bp[a1[a1['SA_Tn']==np.unique(a1['SA_Tn'])[2]].index])); ELR_975b.append(np.mean(ELR_b[a1[a1['SA_Tn']==np.unique(a1['SA_Tn'])[2]].index])); ELR_975o.append(np.mean(ELR_o[a1[a1['SA_Tn']==np.unique(a1['SA_Tn'])[2]].index])); ELR_975m.append(np.mean(ELR_m[a1[a1['SA_Tn']==np.unique(a1['SA_Tn'])[2]].index])); ELR_975mp.append(np.mean(ELR_mp[a1[a1['SA_Tn']==np.unique(a1['SA_Tn'])[2]].index])); 
    ELR_2475up.append(np.mean(ELR_up[a1[a1['SA_Tn']==np.unique(a1['SA_Tn'])[3]].index])) ; ELR_2475up0.append(np.mean(ELR_up0[a1[a1['SA_Tn']==np.unique(a1['SA_Tn'])[3]].index])); ELR_2475bp.append(np.mean(ELR_bp[a1[a1['SA_Tn']==np.unique(a1['SA_Tn'])[3]].index])); ELR_2475b.append(np.mean(ELR_b[a1[a1['SA_Tn']==np.unique(a1['SA_Tn'])[3]].index])); ELR_2475o.append(np.mean(ELR_o[a1[a1['SA_Tn']==np.unique(a1['SA_Tn'])[3]].index])); ELR_2475m.append(np.mean(ELR_m[a1[a1['SA_Tn']==np.unique(a1['SA_Tn'])[3]].index])); ELR_2475mp.append(np.mean(ELR_mp[a1[a1['SA_Tn']==np.unique(a1['SA_Tn'])[3]].index])); 
    ELR_4975up.append(np.mean(ELR_up[a1[a1['SA_Tn']==np.unique(a1['SA_Tn'])[4]].index])) ; ELR_4975up0.append(np.mean(ELR_up0[a1[a1['SA_Tn']==np.unique(a1['SA_Tn'])[4]].index])); ELR_4975bp.append(np.mean(ELR_bp[a1[a1['SA_Tn']==np.unique(a1['SA_Tn'])[4]].index])); ELR_4975b.append(np.mean(ELR_b[a1[a1['SA_Tn']==np.unique(a1['SA_Tn'])[4]].index])); ELR_4975o.append(np.mean(ELR_o[a1[a1['SA_Tn']==np.unique(a1['SA_Tn'])[4]].index])); ELR_4975m.append(np.mean(ELR_m[a1[a1['SA_Tn']==np.unique(a1['SA_Tn'])[4]].index])); ELR_4975mp.append(np.mean(ELR_mp[a1[a1['SA_Tn']==np.unique(a1['SA_Tn'])[4]].index])); 

ELR_o_avg = [np.mean(ELR_100o),np.mean(ELR_475o),np.mean(ELR_975o),np.mean(ELR_2475o),np.mean(ELR_4975o)]
ELR_up_avg = [np.mean(ELR_100up),np.mean(ELR_475up),np.mean(ELR_975up),np.mean(ELR_2475up),np.mean(ELR_4975up)]
ELR_up_avg0 = [np.mean(ELR_100up0),np.mean(ELR_475up0),np.mean(ELR_975up0),np.mean(ELR_2475up0),np.mean(ELR_4975up0)]
ELR_bp_avg = [np.mean(ELR_100bp),np.mean(ELR_475bp),np.mean(ELR_975bp),np.mean(ELR_2475bp),np.mean(ELR_4975bp)]
ELR_b_avg = [np.mean(ELR_100b),np.mean(ELR_475b),np.mean(ELR_975b),np.mean(ELR_2475b),np.mean(ELR_4975b)]
ELR_m_avg = [np.mean(ELR_100m),np.mean(ELR_475m),np.mean(ELR_975m),np.mean(ELR_2475m),np.mean(ELR_4975m)]
ELR_mp_avg = [np.mean(ELR_100mp),np.mean(ELR_475mp),np.mean(ELR_975mp),np.mean(ELR_2475mp),np.mean(ELR_4975mp)]


#%% plot fragility surfaces for collapse damage state

min_x1=0.01; max_x1=1
min_x2=0; max_x2=200

x1 = np.linspace(min_x1, max_x1, 100)
x2 = np.linspace(min_x2, max_x2, 100)
x1, x2 = np.meshgrid(x1, x2)

map_col = sns.color_palette("magma_r", as_cmap=True)
fig, ax = plt.subplots(subplot_kw={"projection": "3d"})
fig.dpi=300
fig.set_size_inches(7, 5)

Z=probit2(np.log(x1),x2)[1][3]
surf = ax.plot_surface(x1, x2, Z, cmap=map_col, linewidth=0, antialiased=False, vmin = 0, vmax =1)
ax.view_init(30, -70)

ax.set_xlabel('$SA_{avg}$ (g)', fontsize=12)
ax.set_ylabel('$D_{s}$ (s)', fontsize=12)
ax.set_zlabel('Probability of Collapse', fontsize=12)
ax.set_title('Collapse Fragility Surface', fontsize=12)
ax.set_zlim(0, 1.0)
ax.set_xlim(0, max_x1)
ax.set_ylim(0, max_x2)
ax.set_yticks([0, 50, 100, 150, 200])

ax.zaxis.set_major_locator(LinearLocator(6))

cbaxes = fig.add_axes([0.13, 0.2, 0.03, 0.6])
cb = fig.colorbar(surf, cax=cbaxes)
cb.set_ticks([0, 0.2, 0.4, 0.6, 0.8, 1])

#%% plot vulnerability surface

min_x1=0.01; max_x1=1
min_x2=0; max_x2=200

x1 = np.linspace(min_x1, max_x1, 101)
x2 = np.linspace(min_x2, max_x2, 101)
x1, x2 = np.meshgrid(x1, x2)
x11 = np.linspace(min_x1, max_x1, 101)

#positions = np.vstack([x1.ravel(), x2.ravel()])
#values = np.vstack([X_input[var1], X_input[var2]])
#kernel = st.gaussian_kde(values)
#f = np.reshape(kernel(positions).T, x1.shape)

Pu=np.squeeze(probit_uni(np.log(x11))[0])
Pp=np.squeeze(probit2(np.log(x1),x2)[0])
P=np.squeeze(p2(np.log(x1),x2)[0])

V=np.zeros((len(x1),len(x2)))
Vp=np.zeros((len(x1),len(x2)))
Vu=np.zeros((len(x1)))

for i in range(len(x1)):
    for j in range(len(x2)):
        V[i,j] = P[0][i,j]*loss_array[0] + P[1][i,j]*loss_array[1] + P[2][i,j]*loss_array[2] + P[3][i,j]*loss_array[3] + P[4][i,j]*loss_array[4]
        Vp[i,j] = Pp[0][i,j]*loss_array[0] + Pp[1][i,j]*loss_array[1] + Pp[2][i,j]*loss_array[2] + Pp[3][i,j]*loss_array[3] + Pp[4][i,j]*loss_array[4]        
        Vu[i] = Pu[0][i]*loss_array[0] + Pu[1][i]*loss_array[1] + Pu[2][i]*loss_array[2] + Pu[3][i]*loss_array[3] + Pu[4][i]*loss_array[4]

fig, ax = plt.subplots(subplot_kw={"projection": "3d"})
fig.dpi=300
#fig.figsize=(5.25,5.25)
fig.set_size_inches(7, 5)
map_col = sns.color_palette("magma_r", as_cmap=True)


surf = ax.plot_surface(x1, x2, Vp, cmap=map_col, linewidth=0, antialiased=False, vmin = 0, vmax =1)

ax.set_xlabel('$SA_{avg}$ (g)', fontsize=12)
ax.set_ylabel('$D_{s}$ (s)', fontsize=12)
ax.set_zlabel('Loss Ratio', fontsize=12)
ax.set_title('Seismic Vulnerability Surface', fontsize=12)
ax.set_zlim(0, 1.0)
ax.set_xlim(0, max_x1)
ax.set_ylim(0, max_x2)
ax.set_yticks([0, 50, 100, 150, 200])
#ax.clim(0, 1)
#ax.view_init(10, 50)
ax.view_init(30, -70)

ax.zaxis.set_major_locator(LinearLocator(6))

cbaxes = fig.add_axes([0.13, 0.2, 0.03, 0.6])
cb = fig.colorbar(surf, cax=cbaxes)
cb.set_ticks([0, 0.2, 0.4, 0.6, 0.8, 1])

#%%
# plot 2d vulnerability & collapse curves

x = np.linspace(0, 1, 101)
colors = [magma(24)[23], magma(24)[20], magma(24)[16], magma(24)[13], magma(24)[3]]

plt.figure(figsize=(3.25,2.5), dpi=300)

plt.plot(x, Vu, linewidth=1.5, color=colors[1], label='U$_{2}$')
plt.plot(x, Vp[10,:], linewidth=1.5, color=colors[2], linestyle='--', label='B$_{n}$ ($D_{s}$ = 20s)')
plt.plot(x, Vp[60,:], linewidth=1.5, color=colors[2], label='B$_{n}$ ($D_{s}$ = 120s)')
plt.xlim(0,1)
plt.ylim(0,1)
plt.xticks([0,0.2,0.4,0.6,0.8,1.0])
plt.legend(fontsize=9, loc='best')
plt.xlabel('$SA_{avg}$ (g)', fontsize=10)
plt.ylabel('Loss Ratio', fontsize=10)
plt.title('Vulnerability Curves', fontsize=10)




#%% conditional probabilities of IMs
#ds_num=3 #exc collapse

### define SA_avg and duration####
#SA_avg = 0.9
#T_dur = 20

min_x1=0.2; max_x1=2
min_x2=0.1; max_x2=.6

x1 = np.linspace(min_x1, max_x1, 10)
x2 = np.linspace(min_x2, max_x2, 6)
x1, x2 = np.meshgrid(x1, x2)

positions = np.vstack([x1.ravel(), x2.ravel()])
values = np.vstack([X_m[var1_m], X_m[var2_m]])
kernel = st.gaussian_kde(values)
f = np.reshape(kernel(positions).T, x1.shape)
ff=f/sum(f)

plt.figure(figsize=(5.5,3), dpi=300)

cmap = sns.cm.rocket_r
x_axis_labels = [0.2,0.4,0.6,0.8,1.0,1.2,1.4,1.6,1.8,2.0]
y_axis_labels = [0.1,0.2,0.3,0.4,0.5,0.6]

ax = sns.heatmap(ff, annot=True, cmap=cmap, vmin=0, vmax=.6, linewidths=.5, fmt='.2f', xticklabels=x_axis_labels, yticklabels=y_axis_labels)
ax.set_yticklabels(ax.get_yticklabels(), rotation = 0)
ax.invert_yaxis()
ax.set_xlabel('$SA_{avg}^{SP}$ (g)', fontsize=11)
ax.set_ylabel('$SA_{avg}^{LP}$ (g)', fontsize=11)
plt.title('Conditional Probability', fontsize=11)
plt.show()


plt.figure(figsize=(3.75,2.25), dpi=300)
sns.histplot(X_m[var2_m],stat="probability", bins=34, color='grey')
plt.xlim(0.0, .6)
plt.ylim(0, .2) 
plt.ylabel('Relative Frequency', fontsize=10)
plt.xlabel('$SA_{avg}^{LP}$ (g)', fontsize=10)



plt.figure(figsize=(3.75,2.25), dpi=300)
sns.histplot(X_m[var1_m],stat="probability", bins=34, color='grey')
plt.xlim(0.0, 2)
plt.ylim(0, .25) 
plt.ylabel('Relative Frequency', fontsize=10)
plt.xlabel('$SA_{avg}^{SP}$ (g)', fontsize=10)
plt.xticks([0,0.2,0.4,0.6,0.8,1.0,1.2,1.4,1.6,1.8,2.0])



#%% plot 2d multivariate collapse fragilities
colors = [magma(24)[23], magma(24)[20], magma(24)[16], magma(24)[13], magma(24)[4]]

plt.figure(figsize=(4.25,2.75), dpi=300)
min_x1=0; max_x1=2
min_x2=0; max_x2=.6

x1 = np.linspace(min_x1, max_x1, 100)
x2 = np.linspace(min_x2, max_x2, 100)
x1, x2 = np.meshgrid(x1, x2)
x=x2[:,0]

T=120
sa1=1.4

plt.plot(x, p3(.4,x,20)[1][3], linewidth=1.5, LineStyle='--', color=colors[4], label='$M_{o}$ $(D_{s} = 20s$ & $SA_{avg}^{SP} = 0.4g$)')
plt.plot(x, p3(sa1,x,T)[1][3], linewidth=1.5, color=colors[4], label='$M_{o}$ $(D_{s} = 120s$ & $SA_{avg}^{SP} = 1.4g$)')
plt.plot(x, probit3(.4,x,20)[1][3], linewidth=1.5, LineStyle='--', color=colors[3], label='$M_{n}$ $(D_{s} = 20s$ & $SA_{avg}^{SP} = 0.4g$)')
plt.plot(x, probit3(sa1,x,T)[1][3], linewidth=1.5, color=colors[3], label='$M_{n}$ $(D_{s} = 120s$ & $SA_{avg}^{SP} = 1.4g$)')

#lay out real data
X_all = pd.concat([X_m, Y],  axis=1)

X_ds =  X_all[(X_all['Ds']<=T*1.05) & (X_all['Ds']>=T*0.95)] #within 5% tolerance
X_sa1 = X_ds[(X_ds['SA_avg_2']<=sa1*1.05) & (X_ds['SA_avg_2']>=sa1*0.95)] #within 5% tolerance

sa2_ds4 = np.mean(X_sa1[X_sa1['Label']==4]['SA_avg_3'])
min_sa2_ds4 = min(X_sa1[X_sa1['Label']==4]['SA_avg_3'])
num_ds4 = len(X_sa1[X_sa1['SA_avg_3']>=min_sa2_ds4])
prob_col_ds4 = len(X_sa1[X_sa1['Label']==4])/ num_ds4


plt.xlim(0,.6)
plt.ylim(0,1)
plt.xticks([0,0.1,0.2,0.3,0.4,0.5,.6])
plt.legend(fontsize=8, loc='best')
plt.xlabel('$SA_{avg}^{LP}$ (g)', fontsize=10)
plt.ylabel('Probability of Collapse', fontsize=10)
plt.title('Multivariate Collapse Fragilities', fontsize=10)
#plt.title('Multivariate Ordinal Fragilities', fontsize=10)



#%% loss & collapse risk bar charts:
    
plt.rcParams["figure.figsize"] = (6,2.75)
plt.rcParams["figure.dpi"] = 400

colors = ['grey',magma(24)[23], magma(24)[20], magma(24)[16], magma(24)[13], magma(24)[4]]

df = pd.DataFrame([['100-yr']+[0.003,2.6,0.4,0.2,0.1,0.6], ['475-yr']+[2.1,10.8,7.5,6.0,3.3,2.9], ['975-yr']+[7.1,15.5,13.9,12.6,8.3,6.8],
                   ['2475-yr']+[24.6,21.5,24.0,24.6,24.2,24.3],['4975-yr']+[40.3,24.9,29.6,31.4,37.4,39.9]],
                  columns=['Hazard Level','Observed', 'U$_{1}$', 'U$_{2}$', 'B$_{n}$', 'M$_{n}$','M$_{o}$'])

dg = df
dg.iloc[:,1:] = df.iloc[:,1:].div(100)

dg.plot(x='Hazard Level',
        kind='bar',
        stacked=False,
        width=.8,rot=0,edgecolor='black',color=colors, legend=False)
plt.ylim(0,.41)
plt.xlabel('Hazard Level',fontsize=12)
plt.ylabel('Expected Loss Ratio',fontsize=12)
#plt.title('Seismic Losses',fontsize=12)
plt.xticks(fontsize=12)
plt.yticks(fontsize=12)
#plt.legend(fontsize=11)

# 100-year only
plt.rcParams["figure.figsize"] = (3,2)
plt.rcParams["figure.dpi"] = 400
df = pd.DataFrame([['100-yr']+[0.003,2.6,0.4,0.2,0.1,0.6]],
                  columns=['Hazard Level','Observed', 'U$_{1}$', 'U$_{2}$', 'B$_{n}$', 'M$_{n}$','M$_{o}$'])
dg = df
dg.iloc[:,1:] = df.iloc[:,1:].div(100)

dg.plot(x='Hazard Level',
        kind='bar',
        stacked=False,
        width=.8,rot=0,edgecolor='black',color=colors,legend=False)
plt.ylim(0,0.03)
plt.xlabel('Hazard Level',fontsize=12)
plt.ylabel('Expected Loss Ratio',fontsize=14)
plt.title('Seismic Losses',fontsize=12)
plt.xticks(fontsize=14)
plt.yticks(fontsize=14)

